/* eslint no-bitwise: "off" */
/**
 * lqApp
 *
 * Events and Utilities used to setup and run the
 *   PhoneGap-based web app inside the native device.
 */
var lqApp = {

  /**
   * bool Ready -- App has connectivity and has handled the DeviceReady
   */
  Ready: false,

  /**
   * bool StartupComplete -- App startup tasks (e.g. Authentication) are complete.
   */
  StartupComplete: false,

  /**
   * @param {boolean} AppPaused - used to track when app is in background
   */
  AppPaused: false,

  /**
   * Int Timestamp -- Timestamp of last app active use.
   */
  Timestamp: null,

  /**
   * Int OfflineTimeout -- timeout for when a user loses connectivity
   */
  OfflineTimeout: null,

  /**
   * Promise StartUpPromise -- Promise used to load app user.
   */
  StartUpPromise: null,

  BootStrapped: false,

  /**
   * init
   *
   * Application Constructor
   */
  Init: function () {
    //
    // Add Device Event listeners
    document.addEventListener('deviceready', lqApp.Events.onDeviceReady, false);
    document.addEventListener('offline', lqApp.Events.onOffline, false);
    document.addEventListener('online', lqApp.Events.onOnline, false);

    //
    // Add DOM Event Listeners
    document.addEventListener('DOMContentLoaded', lqApp.Events.onDOMContentLoaded);
    document.addEventListener('DOMSubtreeModified', lqApp.Events.onDOMSubtreeModified);
  },

  /**
   * Events
   *
   * Handlers for App-level events.
   */
  Events: {

    /**
     * DeviceReady
     *
     * Binds to 'deviceready' event fired by Cordova.
     */
    onDeviceReady: function () {
      if (navigator.hasOwnProperty('connection') && navigator.connection.type == Connection.NONE) {
        navigator.notification.alert('We\'re having trouble connecting to the network. Please check your connection and try again.');
      } else {

        var sessionId;
        navigator.splashscreen.show();
        StatusBar.show();

        if (window.cordova && window.cordova.plugins.Keyboard) {
          cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
        }

        //
        // Init site reload timer.
        lqApp.Util.resetTimestamp();

        //
        // Setup the session ID.
        sessionId = window.sessionStorage.getItem('global.sessionId');
        if (sessionId == '' || sessionId == null) {
          window.sessionStorage.setItem('global.sessionId', lqApp.Util.generateUUID());
        }

        // prevent bootstrap from firing twice
        if (!lqApp.BootStrapped) {
          //
          // Initialize Angular
          angular.bootstrap(document.body, ['lqMobile']);
          lqApp.BootStrapped = true;
        }

        // Mock device.platform property if not available
        if (!window.device) {
          window.device = {platform: 'Browser'};
        }

        //
        // Update app status.
        lqApp.Ready = true;
      }
    },

    /**
     * onOffline
     *
     * Fires when the device loses network connectivity and goes offline.
     */
    onOffline: function () {
      console.log('Device Offline');
    },


    /**
     * onOnline
     *
     * Fires when the device gains network connectivity and comes back online.
     */
    onOnline: function () {
      //
      // if app not properly started
      if (!lqApp.Ready) {
        //
        // Trigger app startup
        lqApp.Events.onDeviceReady();
      }
    },


    /**
     * onDOMContentLoaded
     *
     * Binds to DOMContentLoad event.
     */
    onDOMContentLoaded: function () {
      //
      // Fix images in template
      lqApp.Util.fixImages();
    },

    /**
     * onDOMSubtreeModified
     *
     * Binds to DOMSubtreeModified event.
     */
    onDOMSubtreeModified: function () {
      //
      // Fix images
      lqApp.Util.fixImages();
    }

  }, // END Events


  /**
   * Util
   *
   * App utilities.
   */
  Util: {

    /**
     * resetTimestamp
     *
     * Resets the timeout for the site to reload itself.
     */
    resetTimestamp: function () {
      lqApp.Timestamp = new Date().getTime();
    },


    /**
     * generateUUID
     *
     * Generate a UUID for use in the app.
     *
     * @return String
     */
    generateUUID: function () {
      var d = new Date().getTime();
      var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = (d + (Math.random() * 16)) % 16 | 0;
        d = Math.floor(d / 16);
        return (c == 'x' ? r : ((r & 0x7) | 0x8)).toString(16);
      });
      return uuid;
    },


    /**
     * fixImages
     *
     * Add full URL to images. This allows image assets to be manged in AEM
     * and loaded via URL.
     */
    fixImages: function () {
      var src;
      var images = document.getElementsByTagName('img');
      var i;
      for (i = 0; i < images.length; i += 1) {
        src = images[i].getAttribute('src');
        if (src
          && src.indexOf('data:') != 0
          && src.indexOf('http') != 0
          && src.indexOf('./assets') != 0) {
          images[i].src = lqmobileConfigs.httpBase + src;
        }
        if (src && src.indexOf('//') == 0) {
          images[i].src = 'https:' + src;
        }
      }
    }
  } // END Util

};


/**
 * lqMobileModule.run
 *
 * App startup tasks that will run as soon as the
 * Angular app has been bootstrapped.
 */
lqMobileModule.run(
  ['$rootScope',
    '$modal',
    'routingService',
    'returnsUserProfileModel',
    'returnsUserDeviceModel',
    'freeNightModel',
    'urlService',
    'geolocationService',
    'persistentStorageModelService',
    '$http',
    'satelliteTrackingModelService',
    'touchIdService',
    'localStorageService',
    function (
      $rootScope,
      $modal,
      routingService,
      returnsUserProfileModel,
      returnsUserDeviceModel,
      freeNightModel,
      urlService,
      geolocationService,
      persistentStorageModelService,
      $http,
      satelliteTrackingModelService,
      touchIdService,
      localStorageService
    ) {
      var $localScope = {
        shown: null
      };
      var eligiblityURI = lqmobileConfigs.servicesEndpoint
        + lqmobileConfigs.appVersionCompatibility;

      persistentStorageModelService.getValue('lastLocationSetting').then(
        function (result) {
          $localScope.shown = typeof result !== 'undefined' ? result : null;
        }
      );

      $localScope.showError = function () {
        $localScope.open('Please enable location services in your device settings.', true);
      };

      function freeNightCheck() {
        if (!freeNightModel.built) {
          freeNightModel.checkEligibility();
        }
      }

      /**
       * AppStartup
       *
       * Process all app startup and initialization tasks such as authenticating
       * the user based on stored credentials.
       */
      function AppStartup() {
        lqApp.AppPaused = false;
        var primaryUser = localStorageService.getValue(returnsUserDeviceModel.RETURNS_ACCOUNT);

        navigator.splashscreen.show();
        satelliteTrackingModelService.setAppVersion(lqmobileConfigs.mobileAppVersion);

        // Makes sure all links starting with http are opened externally
        urlService.handleLinksExternally();

        //
        // If user has been set. Setup push notification events.
        //   This is necessary to still receive the event when user is logged out.
        if (primaryUser != null) {
          returnsUserDeviceModel.initPushNotifications(primaryUser);
        }

        //
        // Authenticate the user.
        try {
          returnsUserProfileModel.authenticateUserFromStorage().then(
            //
            // Auth success.
            function () {
              touchIdService.userIsUsingFingerAuth(
                returnsUserProfileModel.data.userProfile.returnsNumber
              ).then(
                function (isUsingData) {
                  if (!isUsingData.isUsing) {
                    touchIdService.hasSeenWizard().then(
                      function (hasSeen) {
                        hasSeen = !!hasSeen;
                        if (!hasSeen) {
                          $rootScope.$broadcast('show-fingerprint-wizard');
                        }
                      },
                      function () {
                        // do nothing
                      }
                    );
                  }
                },
                function () {
                  // do nothing
                }
              );

              // Handle any Push Notifications received.
              returnsUserDeviceModel.processNotificationQueue();
              satelliteTrackingModelService.setLoginStatus(true);
              $rootScope.$broadcast('login-complete');
              navigator.splashscreen.hide();
              lqApp.StartupComplete = true;
            },

            // Auth failure.
            function () {
              navigator.splashscreen.hide();
              lqApp.StartupComplete = true;
              touchIdService.userIsUsingFingerAuth().then(
                function (data) {
                  if (data.isUsing) {
                    touchIdService.promptForFingerAuth().then(
                      function (creds) {
                        var loadingModal = $modal.open({
                          templateUrl: 'loadingModalContent.html',
                          windowClass: 'loading-modal-window',
                          backdrop: 'static',
                          keyboard: false
                        });
                        returnsUserProfileModel.storeCredentials(
                          creds.username, '', creds.lastname, creds.password
                        ).then(
                          function () {
                            returnsUserProfileModel.authenticateUserFromStorage().then(
                              //
                              // Auth success.
                              function () {
                                // Handle any Push Notifications received.
                                returnsUserDeviceModel.processNotificationQueue();
                                satelliteTrackingModelService.setLoginStatus(true);

                                loadingModal.close();
                                $rootScope.$broadcast('login-complete');
                                lqApp.StartupComplete = true;
                              },
                              // Auth failure.
                              function (error) {
                                loadingModal.close();
                                $rootScope.touchIdAuthError = error;
                                routingService.goToSignInPage();
                              }
                            );
                          }
                        );
                      },
                      function (error3) {
                        lqApp.StartupComplete = true;
                        satelliteTrackingModelService.setLoginStatus(false);
                        satelliteTrackingModelService.setMemberID('');
                        satelliteTrackingModelService.setMemberTier('');
                        // Handle any Push Notifications received.
                        returnsUserDeviceModel.processNotificationQueue();
                        // coudln't find creditials, need a way to display the error
                        // to the user still;
                        throw error3;
                      }
                    );
                  }
                },
                function (error4) {
                  lqApp.StartupComplete = true;
                  satelliteTrackingModelService.setLoginStatus(false);
                  satelliteTrackingModelService.setMemberID('');
                  satelliteTrackingModelService.setMemberTier('');
                  // Handle any Push Notifications received.
                  returnsUserDeviceModel.processNotificationQueue();
                  // cannot find touch id, do nothing
                  // throw because everything else does
                  throw error4;
                }
              );
            }
          );
        } catch (err) {
          navigator.splashscreen.hide();
          lqApp.StartupComplete = true;
          touchIdService.userIsUsingFingerAuth().then(
            function (data) {
              if (data.isUsing) {
                touchIdService.promptForFingerAuth().then(
                  function (creds) {
                    var loadingModal = $modal.open({
                      templateUrl: 'loadingModalContent.html',
                      windowClass: 'loading-modal-window',
                      backdrop: 'static',
                      keyboard: false
                    });
                    returnsUserProfileModel.storeCredentials(
                      creds.username, '', creds.lastname, creds.password
                    ).then(
                      function () {
                        returnsUserProfileModel.authenticateUserFromStorage().then(
                          //
                          // Auth success.
                          function () {
                            // Handle any Push Notifications received.
                            returnsUserDeviceModel.processNotificationQueue();
                            satelliteTrackingModelService.setLoginStatus(true);

                            loadingModal.close();
                            $rootScope.$broadcast('login-complete');
                            lqApp.StartupComplete = true;
                          },
                          // Auth failure.
                          function (error) {
                            loadingModal.close();
                            $rootScope.touchIdAuthError = error;
                            routingService.goToSignInPage();
                          }
                        );
                      }
                    );
                  },
                  function () {
                    lqApp.StartupComplete = true;
                    satelliteTrackingModelService.setLoginStatus(false);
                    satelliteTrackingModelService.setMemberID('');
                    satelliteTrackingModelService.setMemberTier('');
                    // Handle any Push Notifications received.
                    returnsUserDeviceModel.processNotificationQueue();
                    // coudln't find creditials, need a way to display the error
                    // to the user still;
                  }
                );
              }
            },
            function () {
              // cannot retrive touch id, do nothing
              lqApp.StartupComplete = true;
              satelliteTrackingModelService.setLoginStatus(false);
              satelliteTrackingModelService.setMemberID('');
              satelliteTrackingModelService.setMemberTier('');
              // Handle any Push Notifications received.
              returnsUserDeviceModel.processNotificationQueue();
            }
          );
        }
      }

      /**
       * DevicePause
       *
       * Fires when the app is moved to the background. E.g. when the user
       * switches apps on their device.
       */
      function DevicePause() {
        console.log('Device Pause');
        lqApp.AppPaused = true;

        freeNightModel.reset();
        //
        // Init site reload timer.
        lqApp.Util.resetTimestamp();
      }


      /**
       * DeviceResume
       *
       * Fires when the app is resumes having been open in  the background.
       */
      function DeviceResume() {
        lqApp.AppPaused = false;
        window.clearTimeout(lqApp.OfflineTimeout);

        //
        // Check timeout.
        var time = new Date().getTime();
        var locationTimeout = localStorageService.getValue('locationDenied');

        console.log('Device Resume');

        if (typeof locationTimeout !== 'undefined' && locationTimeout !== null) {
          if (Date.now() < locationTimeout) {
            // somehthing something time out?
            // lqGeolocationService.setPermissionDenied(true);
          } else {
            localStorageService.removeValue('locationDenied');
          }
        }

        //
        // If paused for 2 hours or more...
        if (time >= lqApp.Timestamp + (2 * 3600 * 1000)) {
          // 2 hours 2 * 3600 * 1000

          lqApp.StartupComplete = false;

          //
          // Refresh the auth token.
          // go through full start up
          window.sessionStorage.removeItem('token');
          AppStartup();
        } else if (time >= lqApp.Timestamp + lqmobileConfigs.tokenTimeout) {
          // paused for more than 20 minutes - re-auth the user
          returnsUserProfileModel.authenticateUserFromStorage().then(
            function () {
              freeNightCheck();
              $rootScope.$broadcast('login-complete');
              $rootScope.$broadcast('check-active-stays');
              navigator.splashscreen.hide();
            },
            function () {
              navigator.splashscreen.hide();
            }
          );
        } else {
          // may have lost this value while backgrounded, we aren't calling startup,
          // so let's reset the value
          satelliteTrackingModelService.setAppVersion(lqmobileConfigs.mobileAppVersion);

          // if we don't do the startup, re login the user
          if (returnsUserProfileModel.isLoggedIn()) {
            freeNightCheck();
            $rootScope.$broadcast('login-complete');
            $rootScope.$broadcast('check-active-stays');
          } else {
            // navigator.splashscreen.show();
            returnsUserProfileModel.authenticateUserFromStorage().then(
              function () {
                freeNightCheck();
                $rootScope.$broadcast('login-complete');
                $rootScope.$broadcast('check-active-stays');
                navigator.splashscreen.hide();
              },
              function () {
                navigator.splashscreen.hide();
              }
            );
          }

          //
          // Handle any push notifications received, even if not refreshing user.
          returnsUserDeviceModel.processNotificationQueue();
        }
        // navigator.splashscreen.hide();
      }


      /**
       * DeviceHasUser
       *
       * Handler for customer "deviceHasUser" event. The LQ app will fire this event
       * when the user is successfully authenticated.
       */
      function DeviceHasUser(ev) {
        //
        // Debug
        console.log('Device User -- Returns No. ' + ev.userProfile.returnsNumber);
      }

      /**
       * Retrieve stored dates if they exist
       */
      function getDecommissionDatesFromStorage() {
        var defer = $.Deferred();

        persistentStorageModelService.getValue('disableEnrollmentsDate').then(
          function (disableEnrollmentsDate) {
            persistentStorageModelService.getValue('appDecommissionDate').then(
              function (appDecommissionDate) {
                defer.resolve({
                  storedDisableEnrollDate: disableEnrollmentsDate,
                  storedDecomDate: appDecommissionDate
                });
              },
              function (msg) {
                console.error(msg);
                defer.resolve({
                  storedDisableEnrollDate: disableEnrollmentsDate,
                  storedDecomDate: false
                });
              }
            );
          },
          function (msg) {
            console.error(msg);
            defer.resolve({
              storedDisableEnrollDate: false,
              storedDecomDate: false
            });
          }
        );

        return defer.promise();
      }

      /**
       * Check the decommission dates to apply appropriate classes
       */
      function checkDecommissionDates() {
        // get dates from storage
        getDecommissionDatesFromStorage().then(function (dateStorage) {
          // if dates are coming from config, use those
          var disableEnrollDate = typeof DISABLE_ENROLLMENTS_DATE !== 'undefined' ? DISABLE_ENROLLMENTS_DATE : false;
          var decomDate = typeof APP_DECOMMISSION_DATE !== 'undefined' ? APP_DECOMMISSION_DATE : false;

          // if no date in config, check storage date
          if (!disableEnrollDate) {
            if (dateStorage.storedDisableEnrollDate) {
              disableEnrollDate = dateStorage.storedDisableEnrollDate;
            } else {
              // not in either, use hard coded
              disableEnrollDate = '4/1/2019';
            }
          }

          // if no date in config, check storage date
          if (!decomDate) {
            if (dateStorage.storedDecomDate) {
              decomDate = dateStorage.storedDecomDate;
            } else {
              // not in either, use hard coded
              decomDate = '4/3/2019';
            }
          }

          // update storage dates
          persistentStorageModelService.setValue('disableEnrollmentsDate', disableEnrollDate).then(
            function () {
              persistentStorageModelService.setValue('appDecommissionDate', decomDate);
            }
          );

          // check dates against current time
          var deviceTime = Date.now();
          if (deviceTime > (new Date(disableEnrollDate)).getTime()) $(document.body).addClass('past-rewards-date');
          if (deviceTime > (new Date(decomDate)).getTime()) $(document.body).addClass('past-decom-date');
        });
      }

      /**
       * open
       *
       * Open the Location Error modal to display an error message to the user.
       *
       * @param String errorMessage
       * @param Boolean showSettings
       */
      $localScope.open = function (errorMessage, showSettings) {
        var now = new Date();
        var showIt = true;
        var timeToShowAgain;
        var modalInstance;

        showSettings = (showSettings == undefined) ? false : showSettings;


        if ($localScope.shown !== false) {
          timeToShowAgain = $localScope.shown + lqmobileConfigs.locationTimeout;
          showIt = timeToShowAgain <= now.getTime();
        }
        //
        // Setup the Location Error Modal.
        if (showIt) {
          persistentStorageModelService.setValue('lastLocationSetting', JSON.stringify(now.getTime()));

          modalInstance = $modal.open({
            templateUrl: 'locationErrorModalContent.html',
            controller: 'ModalInstanceCtrl',
            windowClass: 'app-modal-window',
            resolve: {
              message: function () {
                return errorMessage;
              },
              showSettings: function () {
                return showSettings;
              }
            }
          });

          //
          // ???
          modalInstance.result.then(function () {
            $localScope.shown = now.getTime();
          }, function () {
            $localScope.shown = now.getTime();
          });
          $localScope.shown = now.getTime();
        }
      };

      // check if config is downloaded from AEM, give time if not
      if (!lqmobileConfigs.mobileConfigDownloaded) {
        setTimeout(checkDecommissionDates, 5000);
      } else {
        checkDecommissionDates();
      }

      //
      // Add Device Event Listeners
      document.addEventListener('pause', DevicePause, false);
      document.addEventListener('resume', DeviceResume, false);

      //
      // Custom App Events
      document.addEventListener('deviceHasUser', DeviceHasUser, false);

      eligiblityURI = eligiblityURI.replace('{nativeApp}', function () {
        var deviceType;
        try {
          deviceType = device.platform;
        } catch (e) { // Catch exception if device is not defined.
          deviceType = (navigator.userAgent.toLowerCase().indexOf('android') != -1) ? 'Android' : '';
          deviceType = (navigator.userAgent.toLowerCase().indexOf('iphone') != -1) ? 'iOS' : deviceType;
          deviceType = (navigator.userAgent.toLowerCase().indexOf('ipad') != -1) ? 'iOS' : deviceType;
          deviceType = (navigator.userAgent.toLowerCase().indexOf('ipod') != -1) ? 'iOS' : deviceType;
        }
        return deviceType;
      });

      eligiblityURI = eligiblityURI.replace('{version}', lqmobileConfigs.mobileAppVersion);
      // eligiblityURI = './test/elg.json';
      $http({
        method: 'get',
        url: eligiblityURI,
        params: {},
        headers: {
          'Cache-Control': 'no-cache',
          apikey: lqmobileConfigs.apikey
        }

      }).then(
        // Success
        function (response) {
          if (response.hasOwnProperty('data') && response.data.hasOwnProperty('supported') &&
            (response.data.supported === false || response.data.supported === 'false')) {
            return false;
          }
          return true;
        },
        // Error
        function () {
          return true;
        }
      ).then(function (loadApp) {
        if (loadApp) {
          // Run It!
          AppStartup();
        } else {
          window.location = lqmobileConfigs.httpBase + '/content/lq/lq-mobile-app-wrapper2/en/new-version-available.html';
        }
      });
    }]
);

