/* global lqMobileModule*/
/**
 *  Angular directive to truncate multi-line text to visible height
 *
 *  @param bind (angular bound value to append) REQUIRED
 *  @param ellipsisAppend (string) string to append at end of
 *                    truncated text after ellipsis, can be HTML OPTIONAL
 *  @param ellipsisSymbol (string) string to use as ellipsis, replaces default '...' OPTIONAL
 *  @param ellipsisAppendClick (function) function to call if ellipsisAppend is
 *                              clicked (ellipsisAppend must be clicked) OPTIONAL
 *
 *  @example <p data-ellipsis data-ng-bind="boundData"></p>
 *  @example <p data-ellipsis data-ng-bind="boundData" data-ellipsis-symbol="---"></p>
 *  @example <p data-ellipsis data-ng-bind="boundData" data-ellipsis-append="read more"></p>
 *  @example <p data-ellipsis data-ng-bind="boundData" data-ellipsis-append="read more"
 *              data-ellipsis-append-click="displayFull()"></p>
 *
 */
lqMobileModule.directive('ellipsis', ['$timeout', '$window', function ($timeout, $window) {
  return {
    restrict: 'A',
    scope: {
      ngBind: '=',
      ellipsisAppend: '@',
      ellipsisAppendClick: '&',
      ellipsisSymbol: '@'
    },
    compile: function () {
      return function (scope, element, attributes) {
        var $win = angular.element($window);

        /* Window Resize Variables */
        attributes.lastWindowResizeTime = 0;
        attributes.lastWindowResizeWidth = 0;
        attributes.lastWindowResizeHeight = 0;
        attributes.lastWindowTimeoutEvent = null;
        /* State Variables */
        attributes.isTruncated = false;

        /**
        *  Test if element has overflow of text beyond height or max-height
        *
        *  @param element (DOM object)
        *
        *  @return bool
        *
        */
        function isOverflowed(thisElement) {
          return thisElement[0].scrollHeight > thisElement[0].clientHeight;
        }

        function buildEllipsis() {
          var bindArray;
          var i;
          var ellipsisSymbol;
          var appendString;
          var bindArrayStartingLength;
          var initialMaxHeight;

          if (scope.ngBind) {
            bindArray = scope.ngBind.split(' ');
            i = 0;
            ellipsisSymbol = (typeof (attributes.ellipsisSymbol) !== 'undefined') ? attributes.ellipsisSymbol : '&hellip;';
            appendString = (typeof (scope.ellipsisAppend) !== 'undefined' && scope.ellipsisAppend !== '') ? ellipsisSymbol + '<span>' + scope.ellipsisAppend + '</span>' : ellipsisSymbol;

            attributes.isTruncated = false;
            element.text(scope.ngBind);

            // If text has overflow
            if (isOverflowed(element)) {
              bindArrayStartingLength = bindArray.length;
              initialMaxHeight = element[0].clientHeight;

              element.text(scope.ngBind).html(element.html() + appendString);

              // Set complete text and remove one word at a time, until there is no overflow
              for (; i < bindArrayStartingLength; i += 1) {
                bindArray.pop();
                element.text(bindArray.join(' ')).html(element.html() + appendString);

                if (element[0].scrollHeight < initialMaxHeight || isOverflowed(element) === false) {
                  attributes.isTruncated = true;
                  break;
                }
              }

              // If append string was passed and append click function included
              if (ellipsisSymbol != appendString && typeof (scope.ellipsisAppendClick) !== 'undefined' && scope.ellipsisAppendClick !== '') {
                element.find('span').bind('click touchstart', function () {
                  scope.$apply(scope.ellipsisAppendClick);
                });
              }
            }
          }
        }

         /**
        *  Watchers
        */

           /**
          *  Execute ellipsis truncate on ngBind update
          */
        scope.$watch('ngBind', function () {
          $timeout(function () {
            buildEllipsis();
          });
        });

           /**
          *  Execute ellipsis truncate on ngBind update
          */
        scope.$watch('ellipsisAppend', function () {
          buildEllipsis();
        });

           /**
          *  When window width or height changes - re-init truncation
          */

        function onResize() {
          $timeout.cancel(attributes.lastWindowTimeoutEvent);

          attributes.lastWindowTimeoutEvent = $timeout(function () {
            if (
              attributes.lastWindowResizeWidth != window.innerWidth
                || attributes.lastWindowResizeHeight != window.innerHeight
            ) {
              buildEllipsis();
            }

            attributes.lastWindowResizeWidth = window.innerWidth;
            attributes.lastWindowResizeHeight = window.innerHeight;
          }, 75);
        }

        $win.bind('resize', onResize);

          /**
          * Clean up after ourselves
          */
        scope.$on('$destroy', function () {
          $win.unbind('resize', onResize);
        });
      };
    }
  };
}]);
