/* global lqMobileModule PushNotification */
/* jshint maxparams: 11 */
lqMobileModule.factory(

  'returnsUserDeviceModel',

  ['$q',
    '$filter',
    '$rootScope',
    'returnsUserDeviceEndpointService',
    'routingService',
    'satelliteTrackingModelService',
    'localStorageService',
    function (
      $q,
      $filter,
      $rootScope,
      returnsUserDeviceEndpointService,
      routingService,
      satelliteTrackingModelService,
      localStorageService
    ) {
      'use strict';

      var push;
      var NotificationQueue = [];
      var DeviceModel;
      var CHECKOUT_TYPE = 'mobile-checkout';
      var CHECKIN_TYPE = 'mobile-checkin';
      var UNKNOWN_TYPE = 'unknown';
      var CHECKOUT_REGEX = /check[-\s]+out/gi; // case insensitive regex for "check out" or "check-out"
      var CHECKIN_REGEX = /check[-\s]+in/gi; // case insensitive regex for "check in" or "check-in"

      function getNotificationType(data) {
        // data.message,
        // data.title,
        // data.count,
        // data.sound,
        // data.image,
        // data.additionalData
        if (typeof data.message === 'string') {
          // check message content, make sure its .tolowercase so the regex can match
          if (data.message.search(CHECKOUT_REGEX) !== -1) {
            return CHECKOUT_TYPE;
          } else if (data.message.search(CHECKIN_REGEX) !== -1) {
            return CHECKIN_TYPE;
          }
        }

        return UNKNOWN_TYPE;
      }

      /**
      * Push
      *
      * Methods related to the use of Push Notifications for the LQ app.
      */
      DeviceModel = {

        RETURNS_ACCOUNT: 'lqApp.Push.ReturnsAccount',
        DEVICE: 'lqApp.Push.Device',

        /**
        * initPushNotifications
        *
        * Initiate Push Notifications and register the device for the specified user.
        *
        * @param String returnsNumber
        */
        initPushNotifications: function (returnsNumber) {
        //
        // Only ever allow 1 user, the PRIMARY user.
          var primaryUser = localStorageService.getValue(DeviceModel.RETURNS_ACCOUNT);
          if (primaryUser != null && primaryUser != returnsNumber) {
            console.error('User No. ' + returnsNumber + ' may not subscribe to notifications on this device.');
            return;
          }

          //
          // Store our primary user for future use.
          localStorageService.setValue(DeviceModel.RETURNS_ACCOUNT, returnsNumber);


          //
          // Init Push notification object
          push = PushNotification.init({
            android: {
              senderID: '608065580525',
              sound: false,
              vibrate: false,
              icon: 'ic_stat_laquinta',
              iconColor: '#77b800'
            },
            ios: {
              alert: true,
              badge: true,
              sound: false,
              clearBadge: true
            }
          });

          //
          // Bind to the push events.
          push.on('registration', DeviceModel.onRegistration);
          push.on('notification', DeviceModel.onNotification);
          push.on('error', DeviceModel.onError);

          // Check if user has accepted permissions
          PushNotification.hasPermission(function (permissionData) {
            if (!permissionData.isEnabled) {
              console.log('Push notifications disabled by user');
                    // Un-register device
              push.unregister();

                    // turn off the registration endpoint
              push.off('registration');

                    // re-register it with an empty callback
                    // so if it gets called, there is no error
              push.on('registration', function () {});
            }
          }, function (error) {
            console.log('There was an error' + error);
          }
        );

          push.setApplicationIconBadgeNumber(
          function () {
            console.log('success');
          },
          function () {
            console.log('error');
          },
          0
        );
        },

        /**
        * onRegistration
        *
        * Event handler for the Push Notification registration event.
        *
        * @param {object} data
        */
        onRegistration: function (data) {
          //
          // Setup
          var deviceId = data.registrationId;
          var tokenExists = DeviceModel.hasDeviceToken();

          console.log('Device registration...');
          console.log(deviceId);


          //
          // Check for device, if different unsubscribe so old device ID is not sent message.
          if (tokenExists && DeviceModel.getDeviceToken() != deviceId) {
            //
            // Unregister the OLD
            try {
              DeviceModel.unregisterDevice(DeviceModel.getDeviceToken());
            } catch (e) {
              // do nothing by Tres' request Aug 17 '16
            }

            //
            // Register the NEW
            DeviceModel.registerDevice(deviceId);

          //
          // New Device! Send request to register user.
          } else if (!tokenExists) {
            DeviceModel.registerDevice(deviceId);
          }

          //
          // Save this device token to localStorage
          DeviceModel.setDeviceToken(data.registrationId);
        },

      /**
        * onNotification
        *
        * Event handler for the Push Notification notification received event. This
        * callback will fire when the app is opened by tapping on a notification.
        * Sample object: {
        *        "additionalData": {
        *          "foreground": true,
        *          "coldstart": false
        *        },
        *        "message": "App Debug Test from DT"
        *      }
        *
        * @param Object data
        */
        onNotification: function (data) {
          //
          // Debug!
          console.log('Push Notification received...');
          localStorageService.setValue('_DEBUG_PUSH_NOTIFICATIONS_', data);

          //
          // App open - in foreground
          if (data.additionalData.foreground) {
            // do nothing, call will occur next time user arrives on homepage.

          // App in background or closed
          } else {
            // Add notification to the queue for processing.
            NotificationQueue.push(data);
          }

          if (!data.additionalData.coldstart && !data.additionalData.foreground) {
            satelliteTrackingModelService.pushNotificationTouched(getNotificationType(data));
          }

          //
          // Update Badge!
          push.setApplicationIconBadgeNumber(
            function () {
              console.log('success');
            },
            function () {
              console.log('error');
            },
            NotificationQueue.length
          );

          if (lqApp.StartupComplete) {
            DeviceModel.processNotificationQueue();
          }

          // required to finish push process so iOS will work in background
          // https://github.com/phonegap/phonegap-plugin-push/blob/master/docs/PAYLOAD.md#background-notifications-1
          push.finish(function() {
            console.log("processing of push data is finished");
          }, function() {
            console.log("something went wrong with push.finish for ID = " + data.additionalData['google.message_id']);
          }, data.additionalData['google.message_id']);
        },

        /**
        * onError
        *
        * Event handler for the Push Notification error event.
        *
        * @param Error e
        */
        onError: function (e) {
          console.log('Push Notification Error...');
          console.error(e);
        },

        /**
        * setDeviceToken
        *
        * Store the Device Token in local Storage
        *
        * @param String id
        */
        setDeviceToken: function (id) {
          localStorageService.setValue(DeviceModel.DEVICE, id);
        },

        /**
        * getDeviceToken
        *
        * Retrieve the Device Token from local Storage
        *
        * @return String
        */
        getDeviceToken: function () {
          return localStorageService.getValue(DeviceModel.DEVICE);
        },

        /**
        * hasDeviceToken
        *
        * Check whether or not we have a subscribed device token.
        *
        * @return bool
        */
        hasDeviceToken: function () {
          return (DeviceModel.getDeviceToken() != null);
        },

        /**
        * registerDevice
        *
        * Register device with LQ Web Services
        *
        * @param String deviceId
        */
        registerDevice: function (deviceId) {
          var jsonObj;
          //
          // Retrieve Auth Token
          var authToken = window.sessionStorage.getItem('token');

          //
          // Register device service call.
          if (authToken != null) {
            jsonObj = DeviceModel.registerDeviceJSON(deviceId);
            returnsUserDeviceEndpointService.registerDevice(authToken, jsonObj);
          }
        },

        /**
        * unregisterDevice
        *
        * Unregister device with LQ Web Services
        *
        * @param String deviceId
        */
        unregisterDevice: function (deviceId) {
          var jsonObj;

          //
          // Retrieve Auth Token
          var authToken = window.sessionStorage.getItem('token');

          //
          // Unregister device service call
          if (authToken != null) {
            jsonObj = DeviceModel.registerDeviceJSON(deviceId);
            returnsUserDeviceEndpointService.unregisterDevice(authToken, jsonObj);
          }
        },

        /**
        * registerJSON
        *
        * Build the JSON object to send to Register device endpoint/
        *
        * @param deviceId
        * @return String
        */
        registerDeviceJSON: function (deviceId) {
        //
        // Create JSON payload
          return JSON.stringify({
            DeviceRequest: {
              deviceToken: deviceId,
              deviceType: DeviceModel.getDeviceType()
            }
          });
        },

        /**
        * getDeviceType
        *
        * Retrieve the device via cordova-plugin-device. Use light user agent
        * parsing as a backup.
        *
        * @return String
        */
        getDeviceType: function () {
          var deviceType;
          try {
            deviceType = device.platform;
          } catch (e) { // Catch exception if device is not defined.
            deviceType = (navigator.userAgent.toLowerCase().indexOf('android') != -1) ? 'Android' : deviceType;
            deviceType = (navigator.userAgent.toLowerCase().indexOf('iphone') != -1) ? 'iOS' : deviceType;
            deviceType = (navigator.userAgent.toLowerCase().indexOf('ipad') != -1) ? 'iOS' : deviceType;
            deviceType = (navigator.userAgent.toLowerCase().indexOf('ipod') != -1) ? 'iOS' : deviceType;
          }

          return deviceType;
        },


        /**
        * hasNotifications
        *
        * Return true if app has received one or more notifications since last launch
        *
        * @return Boolean
        */
        hasNotifications: function () {
          return (NotificationQueue.length > 0);
        },


        /**
        * getNotifications
        *
        * Retrieves the queued notifications.
        *
        * @return Object[]
        */
        getNotifications: function () {
          return NotificationQueue;
        },


        /**
        * clearNotifications
        *
        * Clears all notifications.
        */
        clearNotifications: function () {
          NotificationQueue = [];
        },

        /**
        * processNotificationQueue
        *
        * Based on external events, we'll want to process the received push notifications
        * once the app has been properly initialized and user auth has been attempted.
        */
        processNotificationQueue: function () {
          var now = new Date();
          var hour = now.getHours();
          var routingAction = '';

          console.log('process notification queue:');
          console.log(NotificationQueue);

          NotificationQueue.forEach(function (notification) {
            // get type
            var notificationType = getNotificationType(notification);

            switch (notificationType) {
              case CHECKOUT_TYPE:
                satelliteTrackingModelService.pushNotificationCheckOut();
                // handle mobile checkout route
                if (hour < 12) {
                  routingAction = 'goToMobileCheckout';
                } else {
                  routingAction = 'goToCheckoutUnavailable';
                }
                break;
              case CHECKIN_TYPE:
                satelliteTrackingModelService.pushNotificationCheckIn();
                // handle mobile checkin route
                routingAction = 'goToMobileCheckin';
                break;
              default:
                // unknown type, go to app home
                break;
            }
          });

          // route if routing action
          if (routingAction) routingService[routingAction]();

          DeviceModel.clearNotifications();
        }

      };

      return DeviceModel;
    }]);
