/* global lqMobileModule,FingerprintAuth */
/**
 * Created by jeffrey on 1/13/17.
 */
/**
 * Touch Id Service to be used as the main interface for
 * interacting with the device touch id, in a platform independent way
 * this service has both android and iOS touch services as dependencies
 * such that it can choose what platform is currently running, and
 * execute the functionality as expected.
 */
lqMobileModule.factory('touchIdService',
  [
    '$rootScope',
    '$q',
    '$modal',
    'androidTouchIdService',
    'iOsTouchIdService',
    'persistentStorageModelService',
    'secureStorageService',
    'satelliteTrackingModelService',
    'returnsUserProfileModel',
    function (
      $rootScope,
      $q,
      $modal,
      androidTouchIdService,
      iOsTouchIdService,
      persistentStorageModelService,
      secureStorageService,
      satelliteTrackingModelService,
      returnsUserProfileModel
    ) {
      'use strict';

      var thisService = {};
      var deviceType = window.device.platform.toLowerCase();

      /**
       * use this function to determine if touch id is active and available on the device
       * and does not respect whether a user has chosen to use fingerprint authentication
       *
       * @return {Promise<bool>}
       */
      thisService.isTouchIdAvailable = function () {
        var deferred;
        var promise;

        switch (deviceType) {
          case 'android':
            promise = androidTouchIdService.isTouchIdAvailable();
            break;
          case 'ios':
            promise = iOsTouchIdService.isTouchIdAvailable();
            break;
          default:
            deferred = $q.defer();
            deferred.resolve(false);
            promise = deferred.promise;
            break;
        }
        return promise;
      };

      /**
       * function displays the prompt for a fingerprint auth, and
       * returns a bool on whether or not the user has supplied a
       * valid fingerprint
       *
       * @return {Promise<Object>,failure<string>}
       * returns a credintails object:
       *    {
       *      "username": {string},
       *      "lastname": {string},
       *      "password": {string
       *    }
       */
      thisService.promptForFingerAuth = function (iOSPromptText) {
        var deferred = $q.defer();

        if (typeof iOSPromptText === 'undefined' || iOSPromptText === null) {
          iOSPromptText = 'Scan your fingerprint please';
        }

        secureStorageService.getValue('touchIDUsername').then(
          function (userName) {
            secureStorageService.getValue('touchIDLastname').then(
              function (lastName) {
                switch (deviceType) {
                  case 'android':
                    androidTouchIdService.promptForFingerAuth(
                      userName, lastName
                    ).then(
                        function (result) {
                          deferred.resolve(result);
                        },
                        function (msg) {
                          deferred.reject(msg);
                        }
                      );
                    break;
                  case 'ios':
                    iOsTouchIdService.promptForFingerAuth(
                      iOSPromptText, userName, lastName
                    ).then(
                      function (result) {
                        deferred.resolve(result);
                      },
                      function (msg) {
                        deferred.reject(msg);
                      }
                    );
                    break;
                  default:
                    deferred.reject('touch id not supported');
                    break;
                }
              },
              function (error) {
                deferred.reject(error);
              }
            );
          },
          function (err) {
            deferred.reject(err);
          }
        );

        return deferred.promise;
      };

      /**
       * generates and stores all necessary data for a user to
       * register as using finger print authentication,
       * for iOS, this requires no additional steps
       * for android, the user will need to auth with a finger again
       *
       * @return {Promise<bool>,failure<string>}
       */
      thisService.registerWithFingerAuthentication = function (userName, lastName, password) {
        // var promise;
        var deferred = $q.defer();

        switch (deviceType) {
          case 'android':
            secureStorageService.setValue('touchIDUsername', userName);
            secureStorageService.setValue('touchIDLastname', lastName);
            androidTouchIdService.registerWithFingerAuthentication(password).then(
              function (result) {
                thisService.setUsingFingerAuth(true);
                deferred.resolve(result);
              },
              function (fail) {
                thisService.setUsingFingerAuth(false);
                deferred.reject(fail);
              }
            );
            break;
          case 'ios':
            secureStorageService.setValue('touchIDUsername', userName);
            secureStorageService.setValue('touchIDLastname', lastName);
            iOsTouchIdService.registerWithFingerAuthentication(password).then(
              function (result) {
                thisService.setUsingFingerAuth(true).then(
                  function () {
                    deferred.resolve(result);
                  },
                  function () {
                    deferred.reject('could not update preferences');
                  }
                );
              },
              function (fail) {
                thisService.setUsingFingerAuth(false);
                deferred.reject(fail);
              }
            );
            break;
          default:
            deferred.reject('touch id not supported');
            break;
        }

        return deferred.promise;
      };

      /**
       * returns whether or not a user has opt-ed in to fingerprint
       * authentication.
       *
       * @return Promise<bool>
       */
      thisService.userIsUsingFingerAuth = function (currentUserId) {
        var defer = $q.defer();
        var isCurrentUser;

        thisService.isTouchIdAvailable().then(
          function (isAvailable) {
            if (isAvailable) {
              persistentStorageModelService.getValue('userUsesTouchId').then(
                function (usesTouchId) {
                  secureStorageService.getValue('touchIDUsername').then(
                    function (userId) {
                      usesTouchId = !!usesTouchId;
                      if (!currentUserId) {
                        isCurrentUser = false;
                      } else {
                        isCurrentUser = (userId === currentUserId || userId === null);
                      }
                      defer.resolve({ isUsing: usesTouchId, isCurrent: isCurrentUser });
                    },
                    function (err) {
                      console.log(err);
                      // is Using is true, but we don't have a touchIDUser
                      // who can be using it at all, so let's just resolve
                      // all of it to false
                      satelliteTrackingModelService.persistentStorageError();
                      defer.resolve({ isUsing: false, isCurrent: false });
                    }
                  );
                },
                function (error) {
                  console.log(error);
                  defer.resolve({ isUsing: false, isCurrent: false });
                }
              );
            } else {
              defer.reject(false);
            }
          },
          function (e) {
            console.log(e);
            defer.reject(false);
          }
        );

        return defer.promise;
      };

      /**
       * set whether or not a user has opt-ed in to fingerprint authentication
       *
       * @param bool {boolean}
       */
      thisService.setUsingFingerAuth = function (bool) {
        return persistentStorageModelService.setValue('userUsesTouchId', bool);
      };

      /**
       * displays the fingerprint wizard dialog, updates the user preference in
       * file storage, and returns the users response, updates user preference
       *
       * @param accepted function  -- user said yes to using fingerprint auth
       * @param declined function  -- user said no to using fingerprint auth
       */
      thisService.showFingerprintWizard = function (accepted, declined) {
        var returnsNumber;
        if (
          returnsUserProfileModel.hasOwnProperty('data')
          && returnsUserProfileModel.data.hasOwnProperty('userProfile')
          && returnsUserProfileModel.data.userProfile.hasOwnProperty('returnsNumber')
        ) {
          returnsNumber = returnsUserProfileModel.data.userProfile.returnsNumber;
        } else {
          returnsNumber = 'unknown';
        }

        $modal.open({
          animation: false,
          templateUrl: 'fingerprintWizard.html',
          scope: $rootScope.$new(),
          windowClass: 'modal-full ' + deviceType,
          controller: [
            '$scope', '$modalInstance', function ($scope, $modalInstance) {
              $scope.accept = function () {
                thisService.setUsingFingerAuth(true);
                thisService.setIfSeenWizard(true);
                accepted($modalInstance.dismiss);
                satelliteTrackingModelService.fingerprintLoginStateToggled('touch id wizard', true, returnsNumber);
              };
              $scope.decline = function () {
                thisService.setUsingFingerAuth(false);
                thisService.setIfSeenWizard(true);
                declined($modalInstance.dismiss);
                satelliteTrackingModelService.fingerprintLoginStateToggled('touch id wizard', false, returnsNumber);
              };
            }
          ]
        });
      };

       /**
       * returns whether or not the wizard has been displayed on this device
       * this value should be permanently stored on the device, so that the
       * touch ID wizard is only every displayed once per device
       *
       * @returns {Promise<Boolean>}
       */
      thisService.hasSeenWizard = function () {
        var deferred = $q.defer();

        persistentStorageModelService.getValue('hasSeenWizard').then(
          function (hasSeen) {
            deferred.resolve(hasSeen);
          },
          function () {
            deferred.resolve(false);
          }
        );

        return deferred.promise;
      };

      /**
       * Marks the wizard as having been shown on this device, this should
       * be stored permanently per device.
       *
       * @return {Boolean}
       */
      thisService.setIfSeenWizard = function (hasSeen) {
        persistentStorageModelService.setValue('hasSeenWizard', hasSeen);
      };

      /**
       * returns whether or not a user has failed a login previously this session,
       * value should reset when the app is brought back into the foreground and
       * at initial launch
       *
       * @returns {boolean}
       */
      thisService.hasFailedPreviousLogin = function () {};

      thisService.touchIdEnabledMessage = function () {
        var defer = $q.defer();

        switch (deviceType) {
          case 'android':
            navigator.notification.alert(FINGER_AUTH_ENABLED_ANDROID_MESSAGE, function () {
              defer.resolve();
            }, FINGER_AUTH_ENABLED_ANDROID_TITLE);
            break;
          case 'ios':
            navigator.notification.alert(FINGER_AUTH_ENABLED_IOS_MESSAGE, function () {
              defer.resolve();
            }, FINGER_AUTH_ENABLED_IOS_TITLE);
            break;
          default:
            defer.resolve();
            break;
        }

        return defer.promise;
      };

      thisService.touchIdDisabledMessage = function () {
        var defer = $q.defer();

        switch (deviceType) {
          case 'android':
            navigator.notification.alert(FINGER_AUTH_DISABLED_ANDROID_MESSAGE, function () {
              defer.resolve();
            }, FINGER_AUTH_DISABLED_ANDROID_TITLE);
            break;
          case 'ios':
            navigator.notification.alert(FINGER_AUTH_DISABLED_IOS_MESSAGE, function () {
              defer.resolve();
            }, FINGER_AUTH_DISABLED_IOS_TITLE);
            break;
          default:
            defer.resolve();
            break;
        }

        return defer.promise;
      };

      thisService.touchIdFailureMessage = function () {
        var defer = $q.defer();

        switch (deviceType) {
          case 'android':
            navigator.notification.alert(FINGER_AUTH_ENABLED_FAILURE_MESSAGE, function () {
              defer.resolve();
            }, FINGER_AUTH_ENABLED_FAILURE_TITLE);
            break;
          case 'ios':
            // you shouldn't be able to get here
            break;
          default:
            defer.resolve();
            break;
        }

        return defer.promise;
      };


      return thisService;
    }
  ]
);
